/*
 * Click nbfs://nbhost/SystemFileSystem/Templates/Licenses/license-default.txt to change this license
 */

package com.jcg;


import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.*;
import java.util.concurrent.atomic.AtomicInteger;

/**
 *
 * @author omozegieaziegbe
 */

public class RoundRobinLoadBalancer {

    private final List<String> servers;
    private final AtomicInteger currentIndex;

    public RoundRobinLoadBalancer(List<String> servers) {
        if (servers == null || servers.isEmpty()) {
            throw new IllegalArgumentException("Server list cannot be null or empty.");
        }
        this.servers = servers;
        this.currentIndex = new AtomicInteger(0);
    }

    // Thread-safe method to get the next server
    public String getServer() {
        int index = currentIndex.getAndUpdate(i -> (i + 1) % servers.size());
        return servers.get(index);
    }

    // Test the load balancer with multi-threading
    public static void main(String[] args) throws InterruptedException, ExecutionException {
        List<String> serverList = List.of("Server1", "Server2", "Server3", "Server4");
        RoundRobinLoadBalancer loadBalancer = new RoundRobinLoadBalancer(serverList);

        // Create an ExecutorService with a fixed thread pool
        ExecutorService executor = Executors.newFixedThreadPool(4);

        // List to hold Future objects
        List<Future<String>> futures = new ArrayList<>();

        // Submit tasks to simulate multiple concurrent requests
        for (int i = 0; i < 10; i++) {
            Future<String> future = executor.submit(loadBalancer::getServer);
            futures.add(future);
        }

        // Wait for all tasks to complete and print the results
        for (Future<String> future : futures) {
            System.out.println("Redirecting request to: " + future.get());
        }

        // Shut down the executor service
        executor.shutdown();
    }
}